From 880a6d0d20945c6277a16c9599008f064fddf985 Mon Sep 17 00:00:00 2001
From: Wu Jianhua <jianhua.wu@intel.com>
Date: Wed, 23 Feb 2022 22:49:01 +0800
Subject: [PATCH 09/10] avfilter/vf_blend_vulkan: add exclusion blend mode

Use the commands below to test: (href: https://trac.ffmpeg.org/wiki/Blend)

I. make an image for test
ffmpeg -f lavfi -i color=s=256x256,geq=r='H-1-Y':g='H-1-Y':b='H-1-Y' -frames 1 \
-y -pix_fmt yuv420p test.jpg

II. blend in sw
ffmpeg -i test.jpg -vf "split[a][b];[b]transpose[b];[a][b]blend=all_mode=exclusion,\
pseudocolor=preset=turbo" -y exclusion_sw.jpg

III. blend in vulkan
ffmpeg -init_hw_device vulkan -i test.jpg -vf "split[a][b];[b]transpose[b];\
[a]hwupload[a];[b]hwupload[b];[a][b]blend_vulkan=all_mode=exclusion,hwdownload,\
format=yuv420p,pseudocolor=preset=turbo" -y exclusion_vulkan.jpg

Signed-off-by: Wu Jianhua <jianhua.wu@intel.com>
---
 libavfilter/vf_blend_vulkan.c | 3 +++
 1 file changed, 3 insertions(+)

diff --git a/libavfilter/vf_blend_vulkan.c b/libavfilter/vf_blend_vulkan.c
index 558cef7064..d51ee1bcb4 100644
--- a/libavfilter/vf_blend_vulkan.c
+++ b/libavfilter/vf_blend_vulkan.c
@@ -91,6 +91,7 @@ DEFINE_BLEND_MODE(EXTREMITY,  FN(abs(MAX - A - B)))
 DEFINE_BLEND_MODE(DIFFERENCE, FN(abs(A - B)))
 DEFINE_BLEND_MODE(DARKEN,     FN(min(A, B)))
 DEFINE_BLEND_MODE(LIGHTEN,    FN(max(A, B)))
+DEFINE_BLEND_MODE(EXCLUSION,  FN(A + B - 2 * A * B / MAX))
 
 static inline void init_blend_func(FilterParamsVulkan *param)
 {
@@ -110,6 +111,7 @@ static inline void init_blend_func(FilterParamsVulkan *param)
     CASE(DIFFERENCE)
     CASE(DARKEN)
     CASE(LIGHTEN)
+    CASE(EXCLUSION)
     default: param->blend = NULL; break;
     }
 
@@ -489,6 +491,7 @@ static const AVOption blend_vulkan_options[] = {
         { "difference", "", 0, AV_OPT_TYPE_CONST, { .i64 = BLEND_DIFFERENCE }, 0, 0, FLAGS, "mode" },
         { "darken",     "", 0, AV_OPT_TYPE_CONST, { .i64 = BLEND_DARKEN     }, 0, 0, FLAGS, "mode" },
         { "lighten",    "", 0, AV_OPT_TYPE_CONST, { .i64 = BLEND_LIGHTEN    }, 0, 0, FLAGS, "mode" },
+        { "exclusion",  "", 0, AV_OPT_TYPE_CONST, { .i64 = BLEND_EXCLUSION  }, 0, 0, FLAGS, "mode" },
 
     { "c0_opacity",  "set color component #0 opacity", OFFSET(params[0].opacity), AV_OPT_TYPE_DOUBLE, { .dbl = 1 }, 0, 1, FLAGS },
     { "c1_opacity",  "set color component #1 opacity", OFFSET(params[1].opacity), AV_OPT_TYPE_DOUBLE, { .dbl = 1 }, 0, 1, FLAGS },
-- 
2.25.1

